; SetSpeakerVolume.asm
;
; Designed for use from command-line, 
;  batch file or Windows Scheduler.
;
; Usage: SetSpeakerVolume [percent]
;
; LINK with /SUBSYSTEM:WINDOWS
;
; Greg Lyon - 2005
;
.586
.MODEL FLAT, stdcall
option casemap:none

include windows.inc

include kernel32.inc
include user32.inc
include winmm.inc
include masm32.inc
include msvcrt.inc

include c:\masm32\macros\macros.asm ; requires 'macros.asm' updated 7th August 2005

includelib kernel32.lib
includelib user32.lib
includelib winmm.lib
includelib masm32.lib
includelib msvcrt.lib

EndOfString     PROTO pString:PTR BYTE
IsNumericString PROTO pString:PTR BYTE
ConvertPercent  PROTO percent:DWORD
udtoa           PROTO dwValue:DWORD, lpBuffer:PTR BYTE

.CONST

    szTitle         BYTE    "Set Speaker Volume", 0
    szInvalid       BYTE    "Command-line argument is not valid,", 10, "using default value.", 0
    
.DATA?

    MyMixerLine             MIXERLINE                       <?>
    MyMixerControl          MIXERCONTROL                    <?>
    MyMixerLineControls     MIXERLINECONTROLS               <?>
    MyMixerControlDetails   MIXERCONTROLDETAILS             <?>
    MyMixerControlDetails_U MIXERCONTROLDETAILS_UNSIGNED    <?> 

.DATA

    VolumePercent   DWORD    25  ; <<< Set default volume percent here
    hMixer          DWORD     0
    LineID          DWORD     0
    ControlID       DWORD     0
    BassValue       DWORD     0
    szCmdArgBuf     BYTE    128 dup(0)
    szBuf1          BYTE     64 dup(0)
    szBuf2          BYTE     32 dup(0)

.CODE

start: 

    call main
    invoke ExitProcess, eax
    
main PROC
    
    ; If there is a command line argument, use it as volume percentage
    invoke GetCL, 1, ADDR szCmdArgBuf
    .if eax == 1    ;GetCL success
        invoke IsNumericString, ADDR szCmdArgBuf
        .if eax
            mov eax, a2ud(OFFSET szCmdArgBuf)
            mov eax, [eax]
            .if (eax >= 0) && (eax <= 100)
                mov VolumePercent, eax              
            .endif    
        .else    
            invoke MessageBox, HWND_DESKTOP, ADDR szInvalid,  ADDR szTitle, MB_ICONERROR OR MB_TASKMODAL  
        .endif    
    .endif    

    invoke mixerOpen, ADDR hMixer, 0, NULL, NULL, NULL   ; Open preferred Mixer Device (ID = 0)
    .if eax
        invoke udtoa, eax, ADDR szBuf2
        invoke crt_sprintf, ADDR szBuf1, SADD("%s%s"), SADD("Error calling mixerOpen: "), ADDR szBuf2 
        invoke MessageBox, HWND_DESKTOP, ADDR szBuf1,  ADDR szTitle, MB_ICONERROR OR MB_TASKMODAL 
        mov eax, -1
        jmp done
    .endif
    
    mov eax, SIZEOF MIXERLINE
    mov MyMixerLine.cbStruct, eax
    
    mov eax, MIXERLINE_COMPONENTTYPE_DST_SPEAKERS   ; Set device to work with here
    mov MyMixerLine.dwComponentType, eax
    
    invoke mixerGetLineInfo, hMixer, ADDR MyMixerLine, MIXER_GETLINEINFOF_COMPONENTTYPE
    .if eax
        invoke udtoa, eax, ADDR szBuf2
        mov szBuf1[0], 0
        invoke crt_sprintf, ADDR szBuf1, SADD("%s%s"), SADD("Error calling mixerGetLineInfo: "), ADDR szBuf2
        invoke MessageBox, HWND_DESKTOP, ADDR szBuf1, ADDR szTitle, MB_ICONERROR OR MB_TASKMODAL 
        invoke mixerClose, hMixer
        mov eax, -1        
        jmp done
    .endif
    
    mov eax, MyMixerLine.dwLineID
    mov LineID, eax
    
    ; *** Set Volume Control *** 
    mov eax, SIZEOF MIXERLINECONTROLS  
    mov MyMixerLineControls.cbStruct, eax
    
    mov eax, LineID
    mov MyMixerLineControls.dwLineID, eax
    
    mov MyMixerLineControls.cControls, 1
    
    mov eax, MIXERCONTROL_CONTROLTYPE_VOLUME          ; Volume control
    mov MyMixerLineControls.dwControlType, eax
    
    mov eax, OFFSET MyMixerControl
    mov MyMixerLineControls.pamxctrl, eax
    
    mov eax, SIZEOF MIXERCONTROL
    mov MyMixerLineControls.cbmxctrl, eax  
    
    invoke mixerGetLineControls, hMixer, ADDR MyMixerLineControls, MIXER_GETLINECONTROLSF_ONEBYTYPE
     .if eax
        invoke udtoa, eax, ADDR szBuf2
        mov szBuf1[0], 0
        invoke crt_sprintf, ADDR szBuf1, SADD("%s%s"), SADD("Error calling mixerGetLineControls: "), ADDR szBuf2 
        invoke MessageBox, HWND_DESKTOP, ADDR szBuf1, ADDR szTitle, MB_ICONERROR OR MB_TASKMODAL
        invoke mixerClose, hMixer
        mov eax, -1
        jmp done
    .endif
    
    mov eax, MyMixerControl.dwControlID        
    mov ControlID, eax
    
    mov eax, SIZEOF MIXERCONTROLDETAILS
    mov MyMixerControlDetails.cbStruct, eax
    
    mov eax, ControlID  
    mov MyMixerControlDetails.dwControlID, eax
    
    mov MyMixerControlDetails.cChannels, 1
        
    mov MyMixerControlDetails.cMultipleItems, 0
    
    mov eax, OFFSET MyMixerControlDetails_U 
    mov MyMixerControlDetails.paDetails, eax
    
    mov eax, SIZEOF MIXERCONTROLDETAILS_UNSIGNED 
    mov MyMixerControlDetails.cbDetails, eax 

   invoke mixerGetControlDetails, hMixer, ADDR MyMixerControlDetails, MIXER_GETCONTROLDETAILSF_VALUE
   .if eax
        invoke udtoa, eax, ADDR szBuf2
        mov szBuf1[0], 0
        invoke crt_sprintf, ADDR szBuf1, SADD("%s%s"), SADD("Error calling mixerGetControlDetails: "), ADDR szBuf2 
        invoke MessageBox, HWND_DESKTOP, ADDR szBuf1, ADDR szTitle, MB_ICONERROR OR MB_TASKMODAL
        invoke mixerClose, hMixer 
        mov eax, -1
        jmp done
    .endif 
    
    mov eax, MyMixerControlDetails_U.dwValue 
    mov BassValue, eax 
    
    mov eax, SIZEOF MIXERCONTROLDETAILS
    mov MyMixerControlDetails.cbStruct, eax
     
    mov eax, ControlID
    mov MyMixerControlDetails.dwControlID, eax
     
    mov MyMixerControlDetails.cChannels, 1
    
    mov MyMixerControlDetails.cMultipleItems, 0
    
    mov eax, OFFSET MyMixerControlDetails_U
    mov MyMixerControlDetails.paDetails, eax
    
    mov eax, SIZEOF MIXERCONTROLDETAILS_UNSIGNED 
    mov MyMixerControlDetails.cbDetails, eax

    invoke ConvertPercent, VolumePercent                  
    mov MyMixerControlDetails_U.dwValue, eax 
 
    invoke mixerSetControlDetails, hMixer, ADDR MyMixerControlDetails, MIXER_SETCONTROLDETAILSF_VALUE
    .if eax
        invoke udtoa, eax, ADDR szBuf2
        mov szBuf1[0], 0
        invoke crt_sprintf, ADDR szBuf1, SADD("%s%s"), SADD("Error calling mixerSetControlDetails: "), ADDR szBuf2 
        invoke MessageBox, HWND_DESKTOP, ADDR szBuf1, ADDR szTitle, MB_ICONERROR OR MB_TASKMODAL
        invoke mixerClose, hMixer
        mov eax, -1
        jmp done
    .endif 
        
    invoke mixerClose, hMixer
     
    invoke Beep, 2400, 100 
    
done: 

    mov eax, VolumePercent
   
    ret
    
main ENDP    
;---------------------------------------
EndOfString PROC pString:PTR BYTE
    ;returns address of end of string
    mov eax, pString
   @@:
    movzx edx, BYTE PTR [eax]
    .if edx
        inc eax
        jmp @B    
    .endif
    ret
EndOfString ENDP
;---------------------------------------
IsNumericString PROC pString:PTR BYTE
    ; Returns true if a string is numeric (0-9)
    
    invoke EndOfString, pString
    mov ecx, eax
    
    mov eax, pString
    
   @@:
    movzx edx, BYTE PTR [eax]
    
    .if (edx < 48) || (edx > 57)
        jmp  false
    .endif
    
    inc eax
    cmp eax, ecx
    jne @B
    
   true:
    mov eax, 1
    jmp done
    
    false:
     xor eax, eax
     
    done: 
     ret
     
IsNumericString ENDP
;---------------------------------------
ConvertPercent PROC percent:DWORD 
    ; Return in eax = (percent / 100) * 65535 
    ;              or (percent * 65535) / 100
    mov eax, percent
    mov ecx, 65535
    mul ecx
    xor edx, edx
    mov ecx, 100
    div ecx
    ret
ConvertPercent EndP
;---------------------------------------
udtoa PROC udVal:DWORD, lpBuffer:PTR BYTE
    ; udVal is considered unsigned
    invoke crt_sprintf, lpBuffer, SADD("%lu"), udVal
    ret
udtoa ENDP    
;---------------------------------------
end start
